@@ -4,7 +4,7 @@ from .core import istask
 from .context import set_options
 from .async import get_sync as get
 try:
-    from .imperative import do, delayed, value
+    from .delayed import do, delayed, value
 except ImportError:
     pass
 try:
@@ -12,4 +12,4 @@ try:
 except ImportError:
     pass
 
-__version__ = '0.8.2'
+__version__ = '0.10.0'
