@@ -58,8 +58,8 @@ Changing state
     Real-time equivalent of dependents
 
 
-Example
--------
+Examples
+--------
 
 >>> import pprint
 >>> dsk = {'x': 1, 'y': 2, 'z': (inc, 'x'), 'w': (add, 'z', 'y')}
@@ -114,16 +114,18 @@ See the function ``inline_functions`` for more information.
 """
 from __future__ import absolute_import, division, print_function
 
+from operator import add
 import sys
 import traceback
-from operator import add
+
 from .core import (istask, flatten, reverse_dict, get_dependencies, ishashable,
-        _deps)
+                   has_tasks)
 from .context import _globals
 from .order import order
 from .callbacks import unpack_callbacks
 from .optimize import cull
 
+
 def inc(x):
     return x + 1
 
@@ -138,8 +140,8 @@ def start_state_from_dask(dsk, cache=None, sortkey=None):
     --------
 
     >>> dsk = {'x': 1, 'y': 2, 'z': (inc, 'x'), 'w': (add, 'z', 'y')}
-    >>> import pprint
-    >>> pprint.pprint(start_state_from_dask(dsk)) # doctest: +NORMALIZE_WHITESPACE
+    >>> from pprint import pprint
+    >>> pprint(start_state_from_dask(dsk)) # doctest: +NORMALIZE_WHITESPACE
     {'cache': {'x': 1, 'y': 2},
      'dependencies': {'w': set(['y', 'z']),
                       'x': set([]),
@@ -166,7 +168,7 @@ def start_state_from_dask(dsk, cache=None, sortkey=None):
         cache = dict()
     data_keys = set()
     for k, v in dsk.items():
-        if not (istask(v) or _deps(dsk, v)):
+        if not has_tasks(dsk, v):
             cache[k] = v
             data_keys.add(k)
 
@@ -210,6 +212,7 @@ When we execute tasks we both
 2.  Manage administrative state to coordinate with the scheduler
 '''
 
+
 def _execute_task(arg, cache, dsk=None):
     """ Do the actual work of collecting data and executing a function
 
@@ -318,7 +321,7 @@ def finish_task(dsk, key, state, results, sortkey, delete=True,
                 if DEBUG:
                     from chest.core import nbytes
                     print("Key: %s\tDep: %s\t NBytes: %.2f\t Release" % (key, dep,
-                        sum(map(nbytes, state['cache'].values()) / 1e6)))
+                          sum(map(nbytes, state['cache'].values()) / 1e6)))
                 release_data(dep, state, delete=delete)
         elif delete and dep not in results:
             release_data(dep, state, delete=delete)
@@ -371,6 +374,7 @@ this first-in-first-out policy reduces memory footprint
 The main function of the scheduler.  Get is the main entry point.
 '''
 
+
 def get_async(apply_async, num_workers, dsk, result, cache=None,
               queue=None, get_id=default_get_id, raise_on_exception=False,
               rerun_exceptions_locally=None, callbacks=None, **kwargs):
@@ -381,10 +385,6 @@ def get_async(apply_async, num_workers, dsk, result, cache=None,
     specific ``get`` method that walks through the dask array with parallel
     workers, avoiding repeat computation and minimizing memory use.
 
-    This function evaluates the entire graph, regardless of the given output
-    keys.  You may want to cull your graph ahead of time with
-    ``dask.optimize.cull``.
-
     Parameters
     ----------
 
@@ -418,7 +418,7 @@ def get_async(apply_async, num_workers, dsk, result, cache=None,
 
     if callbacks is None:
         callbacks = _globals['callbacks']
-    start_cbs, pretask_cbs, posttask_cbs, finish_cbs = unpack_callbacks(callbacks)
+    start_cbs, start_state_cbs, pretask_cbs, posttask_cbs, finish_cbs = unpack_callbacks(callbacks)
 
     if isinstance(result, list):
         result_flat = set(flatten(result))
@@ -430,12 +430,15 @@ def get_async(apply_async, num_workers, dsk, result, cache=None,
     for f in start_cbs:
         f(dsk)
 
-    dsk = cull(dsk, list(results))
+    dsk, dependencies = cull(dsk, list(results))
 
     keyorder = order(dsk)
 
     state = start_state_from_dask(dsk, cache=cache, sortkey=keyorder.get)
 
+    for f in start_state_cbs:
+        f(dsk, state)
+
     if rerun_exceptions_locally is None:
         rerun_exceptions_locally = _globals.get('rerun_exceptions_locally', False)
 
@@ -558,13 +561,13 @@ class RemoteException(Exception):
     def __str__(self):
         return (str(self.exception) + "\n\n"
                 "Traceback\n"
-                "---------\n"
-                + self.traceback)
+                "---------\n" +
+                self.traceback)
 
     def __dir__(self):
-        return sorted(set(dir(type(self))
-                        + list(self.__dict__)
-                        + dir(self.exception)))
+        return sorted(set(dir(type(self)) +
+                      list(self.__dict__) +
+                      dir(self.exception)))
 
     def __getattr__(self, key):
         try:
