@@ -1,7 +1,8 @@
 from __future__ import absolute_import, division, print_function
 
 from .core import (Bag, Item, from_sequence, from_filenames, from_url,
-                   to_textfiles, concat, from_s3, from_castra, from_imperative,
-                   bag_range as range, bag_zip as zip)
+                   to_textfiles, concat, from_castra, from_imperative,
+                   from_delayed, bag_range as range, bag_zip as zip)
+from .text import read_text
 from ..context import set_options
 from ..base import compute
