@@ -4,30 +4,39 @@ from collections import Iterable, Iterator, defaultdict
 from functools import wraps, partial
 import itertools
 import math
+from operator import getitem
 import os
+import types
 import uuid
 from warnings import warn
+from distutils.version import LooseVersion
 
 from ..utils import ignoring
 
 from toolz import (merge, take, reduce, valmap, map, partition_all, filter,
-        remove, compose, curry)
+                   remove, compose, curry, first, second, accumulate)
 from toolz.compatibility import iteritems, zip
 import toolz
+_implement_accumulate = LooseVersion(toolz.__version__) > '0.7.4'
 try:
+    import cytoolz
     from cytoolz import (frequencies, merge_with, join, reduceby,
                          count, pluck, groupby, topk)
+    if LooseVersion(cytoolz.__version__) > '0.7.3':
+        from cytoolz import accumulate
+        _implement_accumulate = True
 except:
     from toolz import (frequencies, merge_with, join, reduceby,
-                         count, pluck, groupby, topk)
+                       count, pluck, groupby, topk)
 
 from ..base import Base, normalize_token, tokenize
 from ..compatibility import apply, unicode, urlopen
+from ..context import _globals
 from ..core import list2, quote, istask, get_dependencies, reverse_dict
 from ..multiprocessing import get as mpget
 from ..optimize import fuse, cull, inline
 from ..utils import (infer_compression, open, system_encoding,
-                     takes_multiple_arguments, funcname)
+                     takes_multiple_arguments, funcname, digit, insert)
 
 no_default = '__no__default__'
 
@@ -137,6 +146,14 @@ def to_textfiles(b, path, name_function=str, compression='infer',
 
     **Compression**: Filenames with extensions corresponding to known
     compression algorithms (gz, bz2) will be compressed accordingly.
+    
+    **Bag Contents**: The bag calling ``to_textfiles`` _must_ be a bag of
+    text strings. For example, a bag of dictionaries could be written to 
+    JSON text files by mapping ``json.dumps``on to the bag first, and 
+    then calling ``to_textfiles``:
+
+    >>> b_dict.map(json.dumps).to_textfiles("/path/to/data/*.json")  # doctest: +SKIP
+
     """
     if isinstance(path, (str, unicode)):
         if '*' in path:
@@ -320,11 +337,11 @@ class Bag(Base):
         >>> list(b.map(lambda x: x * 10))  # doctest: +SKIP
         [0, 10, 20, 30, 40]
 
-        Keyword arguments are passed through to `func`. These can be either
-        `dask.bag.Item`s, or normal python objects.
+        Keyword arguments are passed through to ``func``. These can be either
+        ``dask.bag.Item``, or normal python objects.
 
-        Example
-        -------
+        Examples
+        --------
         >>> import dask.bag as db
         >>> b = db.from_sequence(range(1, 101), npartitions=10)
         >>> def div(num, den=1):
@@ -338,7 +355,7 @@ class Bag(Base):
         >>> b.map(div, den=hi).take(5)
         (0.01, 0.02, 0.03, 0.04, 0.05)
 
-        Using an `Item`:
+        Using an ``Item``:
 
         >>> b.map(div, den=b.max()).take(5)
         (0.01, 0.02, 0.03, 0.04, 0.05)
@@ -406,11 +423,11 @@ class Bag(Base):
 
         >>> b.map_partitions(myfunc)  # doctest: +SKIP
 
-        Keyword arguments are passed through to `func`. These can be either
-        `dask.bag.Item`s, or normal python objects.
+        Keyword arguments are passed through to ``func``. These can be either
+        ``dask.bag.Item``, or normal python objects.
 
-        Example
-        -------
+        Examples
+        --------
         >>> import dask.bag as db
         >>> b = db.from_sequence(range(1, 101), npartitions=10)
         >>> def div(nums, den=1):
@@ -424,7 +441,7 @@ class Bag(Base):
         >>> b.map_partitions(div, den=hi).take(5)
         (0.01, 0.02, 0.03, 0.04, 0.05)
 
-        Using an `Item`:
+        Using an ``Item``:
 
         >>> b.map_partitions(div, den=b.max()).take(5)
         (0.01, 0.02, 0.03, 0.04, 0.05)
@@ -465,6 +482,24 @@ class Bag(Base):
                        for i in range(self.npartitions))
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
+    def unzip(self, n):
+        """Transform a bag of tuples to ``n`` bags of their elements.
+
+        Examples
+        --------
+        >>> b = from_sequence([(i, i + 1, i + 2) for i in range(10)])
+        >>> first, second, third = b.unzip(3)
+        >>> isinstance(first, Bag)
+        True
+        >>> first.compute()
+        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
+
+        Note that this is equivalent to:
+
+        >>> first, second, third = (b.pluck(i) for i in range(3))
+        """
+        return tuple(self.pluck(i) for i in range(n))
+
     @wraps(to_textfiles)
     def to_textfiles(self, path, name_function=str, compression='infer',
                      encoding=system_encoding, compute=True):
@@ -854,54 +889,56 @@ class Bag(Base):
     def __iter__(self):
         return iter(self.compute())
 
-    def groupby(self, grouper, npartitions=None, blocksize=2**20):
+    def groupby(self, grouper, method=None, npartitions=None, blocksize=2**20,
+                max_branch=None):
         """ Group collection by key function
 
-        Note that this requires full dataset read, serialization and shuffle.
+        This requires a full dataset read, serialization and shuffle.
         This is expensive.  If possible you should use ``foldby``.
 
+        Parameters
+        ----------
+        grouper: function
+            Function on which to group elements
+        method: str
+            Either 'disk' for an on-disk shuffle or 'tasks' to use the task
+            scheduling framework.  Use 'disk' if you are on a single machine
+            and 'tasks' if you are on a distributed cluster.
+        npartitions: int
+            If using the disk-based shuffle, the number of output partitions
+        blocksize: int
+            If using the disk-based shuffle, the size of shuffle blocks
+        max_branch: int
+            If using the task-based shuffle, the amount of splitting each
+            partition undergoes.  Increase this for fewer copies but more
+            scheduler overhead.
+
+        Examples
+        --------
         >>> b = from_sequence(range(10))
-        >>> dict(b.groupby(lambda x: x % 2 == 0))  # doctest: +SKIP
+        >>> iseven = lambda x: x % 2 == 0
+        >>> dict(b.groupby(iseven))  # doctest: +SKIP
         {True: [0, 2, 4, 6, 8], False: [1, 3, 5, 7, 9]}
 
         See Also
         --------
-
         Bag.foldby
         """
-        if npartitions is None:
-            npartitions = self.npartitions
-        token = tokenize(self, grouper, npartitions, blocksize)
-
-        import partd
-        p = ('partd-' + token,)
-        try:
-            dsk1 = {p: (partd.Python, (partd.Snappy, partd.File()))}
-        except AttributeError:
-            dsk1 = {p: (partd.Python, partd.File())}
-
-        # Partition data on disk
-        name = 'groupby-part-{0}-{1}'.format(funcname(grouper), token)
-        dsk2 = dict(((name, i), (partition, grouper, (self.name, i),
-                                 npartitions, p, blocksize))
-                    for i in range(self.npartitions))
-
-        # Barrier
-        barrier_token = 'groupby-barrier-' + token
-
-        def barrier(args):
-            return 0
-
-        dsk3 = {barrier_token: (barrier, list(dsk2))}
-
-        # Collect groups
-        name = 'groupby-collect-' + token
-        dsk4 = dict(((name, i),
-                     (collect, grouper, i, p, barrier_token))
-                    for i in range(npartitions))
-
-        return type(self)(merge(self.dask, dsk1, dsk2, dsk3, dsk4), name,
-                          npartitions)
+        if method is None:
+            get = _globals.get('get')
+            if (isinstance(get, types.MethodType) and
+                'distributed' in get.__func__.__module__):
+                method = 'tasks'
+            else:
+                method = 'disk'
+        if method == 'disk':
+            return groupby_disk(self, grouper, npartitions=npartitions,
+                                blocksize=blocksize)
+        elif method == 'tasks':
+            return groupby_tasks(self, grouper, max_branch=max_branch)
+        else:
+            raise NotImplementedError(
+                    "Shuffle method must be 'disk' or 'tasks'")
 
     def to_dataframe(self, columns=None):
         """ Convert Bag to dask.dataframe
@@ -980,6 +1017,50 @@ class Bag(Base):
                    for i in range(npartitions))
         return Bag(merge(self.dask, dsk), name, npartitions)
 
+    def accumulate(self, binop, initial=no_default):
+        """Repeatedly apply binary function to a sequence, accumulating results.
+
+        Examples
+        --------
+        >>> from operator import add
+        >>> b = from_sequence([1, 2, 3, 4, 5], npartitions=2)
+        >>> b.accumulate(add).compute()  # doctest: +SKIP
+        [1, 3, 6, 10, 15]
+
+        Accumulate also takes an optional argument that will be used as the
+        first value.
+
+        >>> b.accumulate(add, -1)  # doctest: +SKIP
+        [-1, 0, 2, 5, 9, 15]
+        """
+        if not _implement_accumulate:
+            raise NotImplementedError("accumulate requires `toolz` > 0.7.4"
+                                      " or `cytoolz` > 0.7.3.")
+        token = tokenize(self, binop, initial)
+        binop_name = funcname(binop)
+        a = '%s-part-%s' % (binop_name, token)
+        b = '%s-first-%s' % (binop_name, token)
+        c = '%s-second-%s' % (binop_name, token)
+        dsk = {(a, 0): (accumulate_part, binop, (self.name, 0), initial, True),
+            (b, 0): (first, (a, 0)),
+            (c, 0): (second, (a, 0))}
+        for i in range(1, self.npartitions):
+            dsk[(a, i)] = (accumulate_part, binop, (self.name, i),
+                        (c, i - 1))
+            dsk[(b, i)] = (first, (a, i))
+            dsk[(c, i)] = (second, (a, i))
+        return Bag(merge(self.dask, dsk), b, self.npartitions)
+
+
+def accumulate_part(binop, seq, initial, is_first=False):
+    if initial == no_default:
+        res = list(accumulate(binop, seq))
+    else:
+        res = list(accumulate(binop, seq, initial=initial))
+    if is_first:
+        return res, res[-1] if res else [], initial
+    return res[1:], res[-1]
+
 
 normalize_token.register(Item, lambda a: a.key)
 normalize_token.register(Bag, lambda a: a.name)
@@ -1021,7 +1102,11 @@ def write(data, filename, compression, encoding):
 
     # Check presence of endlines
     data = iter(data)
-    firstline = next(data)
+    try:
+        firstline = next(data)
+    except StopIteration:
+        f.close()
+        return
     if not (firstline.endswith(os.linesep) or firstline.endswith('\n')):
         sep = os.linesep if firstline.endswith(os.linesep) else '\n'
         firstline = firstline + sep
@@ -1135,15 +1220,15 @@ def from_url(urls):
     >>> a.npartitions  # doctest: +SKIP
     1
 
-    >> a.take(8)  # doctest: +SKIP
-    ('Dask\n',
-     '====\n',
-     '\n',
-     '|Build Status| |Coverage| |Doc Status| |Gitter|\n',
-     '\n',
-     'Dask provides multi-core execution on larger-than-memory datasets using blocked\n',
-     'algorithms and task scheduling.  It maps high-level NumPy and list operations\n',
-     'on large datasets on to graphs of many operations on small in-memory datasets.\n')
+    >>> a.take(8)  # doctest: +SKIP
+    ('Dask\\n',
+     '====\\n',
+     '\\n',
+     '|Build Status| |Coverage| |Doc Status| |Gitter|\\n',
+     '\\n',
+     'Dask provides multi-core execution on larger-than-memory datasets using blocked\\n',
+     'algorithms and task scheduling.  It maps high-level NumPy and list operations\\n',
+     'on large datasets on to graphs of many operations on small in-memory datasets.\\n')
 
     >>> b = from_url(['http://github.com', 'http://google.com'])  # doctest: +SKIP
     >>> b.npartitions  # doctest: +SKIP
@@ -1361,7 +1446,7 @@ def bag_zip(*bags):
     When what you really wanted was more along the lines of:
     >>> list(fizzbuzzz) # doctest: +SKIP
     [(0, 0), (3, None), (None, 5), (6, None), (None 10), (9, None),
-     (12, None), (15, 15), (18, None), (None, 20), (None, 25), (None, 30)]
+    (12, None), (15, 15), (18, None), (None, 20), (None, 25), (None, 30)]
     """
     npartitions = bags[0].npartitions
     assert all(bag.npartitions == npartitions for bag in bags)
@@ -1380,3 +1465,101 @@ def _reduce(binop, sequence, initial=no_default):
         return reduce(binop, sequence, initial)
     else:
         return reduce(binop, sequence)
+
+
+def make_group(k, stage):
+    def h(x):
+        return x[0] // k ** stage % k
+    return h
+
+
+def groupby_tasks(b, grouper, hash=hash, max_branch=32):
+    max_branch = max_branch or 32
+    n = b.npartitions
+
+    stages = int(math.ceil(math.log(n) / math.log(max_branch)))
+    if stages > 1:
+        k = int(math.ceil(n ** (1 / stages)))
+    else:
+        k = n
+
+    groups = []
+    splits = []
+    joins = []
+
+    inputs = [tuple(digit(i, j, k) for j in range(stages))
+              for i in range(k**stages)]
+    sinputs = set(inputs)
+
+    b2 = b.map(lambda x: (hash(grouper(x)), x))
+
+    token = tokenize(b, grouper, hash, max_branch)
+
+    start = dict((('shuffle-join-' + token, 0, inp),
+                  (b2.name, i) if i < b.npartitions else [])
+                 for i, inp in enumerate(inputs))
+
+    for stage in range(1, stages + 1):
+        group = dict((('shuffle-group-' + token, stage, inp),
+                      (groupby,
+                        (make_group, k, stage - 1),
+                        ('shuffle-join-' + token, stage - 1, inp)))
+                 for inp in inputs)
+
+        split = dict((('shuffle-split-' + token, stage, i, inp),
+                      (dict.get, ('shuffle-group-' + token, stage, inp), i, {}))
+                     for i in range(k)
+                     for inp in inputs)
+
+        join = dict((('shuffle-join-' + token, stage, inp),
+                     (list, (toolz.concat,
+                        [('shuffle-split-' + token, stage, inp[stage-1],
+                          insert(inp, stage - 1, j)) for j in range(k)])))
+                     for inp in inputs)
+        groups.append(group)
+        splits.append(split)
+        joins.append(join)
+
+    end = dict((('shuffle-' + token, i),
+                (list, (dict.items, (groupby, grouper, (pluck, 1, j)))))
+               for i, j in enumerate(join))
+
+    dsk = merge(b2.dask, start, end, *(groups + splits + joins))
+
+    return type(b)(dsk, 'shuffle-' + token, len(inputs))
+
+
+def groupby_disk(b, grouper, npartitions=None, blocksize=2**20):
+    if npartitions is None:
+        npartitions = b.npartitions
+    token = tokenize(b, grouper, npartitions, blocksize)
+
+    import partd
+    p = ('partd-' + token,)
+    try:
+        dsk1 = {p: (partd.Python, (partd.Snappy, partd.File()))}
+    except AttributeError:
+        dsk1 = {p: (partd.Python, partd.File())}
+
+    # Partition data on disk
+    name = 'groupby-part-{0}-{1}'.format(funcname(grouper), token)
+    dsk2 = dict(((name, i), (partition, grouper, (b.name, i),
+                             npartitions, p, blocksize))
+                for i in range(b.npartitions))
+
+    # Barrier
+    barrier_token = 'groupby-barrier-' + token
+
+    def barrier(args):
+        return 0
+
+    dsk3 = {barrier_token: (barrier, list(dsk2))}
+
+    # Collect groups
+    name = 'groupby-collect-' + token
+    dsk4 = dict(((name, i),
+                 (collect, grouper, i, p, barrier_token))
+                for i in range(npartitions))
+
+    return type(b)(merge(b.dask, dsk1, dsk2, dsk3, dsk4), name,
+                         npartitions)
