@@ -4,7 +4,7 @@ from __future__ import absolute_import, division, print_function
 import pytest
 
 from toolz import (merge, join, pipe, filter, identity, merge_with, take,
-        partial, valmap)
+        partial, valmap, groupby, pluck)
 import math
 from dask.bag.core import (Bag, lazify, lazify_task, fuse, map, collect,
         reduceby, reify, partition, inline_singleton_lists, optimize,
@@ -19,10 +19,11 @@ import io
 import shutil
 import os
 import partd
-from tempfile import mkdtemp
 
 from collections import Iterator
 
+from dask.utils import tmpdir
+
 dsk = {('x', 0): (range, 5),
        ('x', 1): (range, 5),
        ('x', 2): (range, 5)}
@@ -158,6 +159,15 @@ def test_pluck_with_default():
     assert b.pluck(0).name != b.pluck(0, None).name
 
 
+def test_unzip():
+    b = db.from_sequence(range(100)).map(lambda x: (x, x + 1, x + 2))
+    one, two, three = b.unzip(3)
+    assert list(one) == list(range(100))
+    assert list(three) == [i + 2 for i in range(100)]
+    assert one.name == b.unzip(3)[0].name
+    assert one.name != two.name
+
+
 def test_fold():
     c = b.fold(add)
     assert c.compute() == sum(L)
@@ -623,11 +633,10 @@ def test_to_dataframe():
 
 def test_to_textfiles():
     b = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
-    dir = mkdtemp()
     for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
-        c = b.to_textfiles(os.path.join(dir, '*.' + ext), compute=False)
-        assert c.npartitions == b.npartitions
-        try:
+        with tmpdir() as dir:
+            c = b.to_textfiles(os.path.join(dir, '*.' + ext), compute=False)
+            assert c.npartitions == b.npartitions
             c.compute(get=dask.get)
             assert os.path.exists(os.path.join(dir, '1.' + ext))
 
@@ -637,18 +646,14 @@ def test_to_textfiles():
                 text = text.decode()
             assert 'xyz' in text
             f.close()
-        finally:
-            if os.path.exists(dir):
-                shutil.rmtree(dir)
 
 
 def test_to_textfiles_encoding():
     b = db.from_sequence([u'汽车', u'苹果', u'天气'], npartitions=2)
-    dir = mkdtemp()
     for ext, myopen in [('gz', GzipFile), ('bz2', BZ2File), ('', open)]:
-        c = b.to_textfiles(os.path.join(dir, '*.' + ext), encoding='gb18030', compute=False)
-        assert c.npartitions == b.npartitions
-        try:
+        with tmpdir() as dir:
+            c = b.to_textfiles(os.path.join(dir, '*.' + ext), encoding='gb18030', compute=False)
+            assert c.npartitions == b.npartitions
             c.compute(get=dask.get)
             assert os.path.exists(os.path.join(dir, '1.' + ext))
 
@@ -658,9 +663,6 @@ def test_to_textfiles_encoding():
                 text = text.decode('gb18030')
             assert u'天气' in text
             f.close()
-        finally:
-            if os.path.exists(dir):
-                shutil.rmtree(dir)
 
 
 def test_to_textfiles_inputs():
@@ -828,3 +830,86 @@ def test_repartition():
         b.repartition(100)
     except NotImplementedError as e:
         assert '100' in str(e)
+
+@pytest.mark.skipif('not db.core._implement_accumulate')
+def test_accumulate():
+    parts = [[1, 2, 3], [4, 5], [], [6, 7]]
+    dsk = dict((('test', i), p) for (i, p) in enumerate(parts))
+    b = db.Bag(dsk, 'test', len(parts))
+    r = b.accumulate(add)
+    assert r.name == b.accumulate(add).name
+    assert r.name != b.accumulate(add, -1).name
+    assert r.compute() == [1, 3, 6, 10, 15, 21, 28]
+    assert b.accumulate(add, -1).compute() == [-1, 0, 2, 5, 9, 14, 20, 27]
+    assert b.accumulate(add).map(inc).compute() == [2, 4, 7, 11, 16, 22, 29]
+
+    b = db.from_sequence([1, 2, 3], npartitions=1)
+    assert b.accumulate(add).compute() == [1, 3, 6]
+
+
+def test_groupby_tasks():
+    b = db.from_sequence(range(160), npartitions=4)
+    out = b.groupby(lambda x: x % 10, max_branch=4, method='tasks')
+    partitions = dask.get(out.dask, out._keys())
+
+    for a in partitions:
+        for b in partitions:
+            if a is not b:
+                assert not set(pluck(0, a)) & set(pluck(0, b))
+
+
+    b = db.from_sequence(range(1000), npartitions=100)
+    out = b.groupby(lambda x: x % 123, method='tasks')
+    assert len(out.dask) < 100**2
+    partitions = dask.get(out.dask, out._keys())
+
+    for a in partitions:
+        for b in partitions:
+            if a is not b:
+                assert not set(pluck(0, a)) & set(pluck(0, b))
+
+
+    b = db.from_sequence(range(10000), npartitions=345)
+    out = b.groupby(lambda x: x % 2834, max_branch=24, method='tasks')
+    partitions = dask.get(out.dask, out._keys())
+
+    for a in partitions:
+        for b in partitions:
+            if a is not b:
+                assert not set(pluck(0, a)) & set(pluck(0, b))
+
+
+def test_groupby_tasks_names():
+    b = db.from_sequence(range(160), npartitions=4)
+    func = lambda x: x % 10
+    func2 = lambda x: x % 20
+    assert (set(b.groupby(func, max_branch=4, method='tasks').dask) ==
+            set(b.groupby(func, max_branch=4, method='tasks').dask))
+    assert (set(b.groupby(func, max_branch=4, method='tasks').dask) !=
+            set(b.groupby(func, max_branch=2, method='tasks').dask))
+    assert (set(b.groupby(func, max_branch=4, method='tasks').dask) !=
+            set(b.groupby(func2, max_branch=4, method='tasks').dask))
+
+
+@pytest.mark.parametrize('size,npartitions,groups', [(1000, 20, 100),
+                                                     (12345, 234, 1042)])
+def test_groupby_tasks_2(size, npartitions, groups):
+    func = lambda x: x % groups
+    b = db.range(size, npartitions=npartitions).groupby(func, method='tasks')
+    result = b.compute(get=dask.get)
+    assert dict(result) == groupby(func, range(size))
+
+
+def test_groupby_tasks_3():
+    func = lambda x: x % 10
+    b = db.range(20, npartitions=5).groupby(func, method='tasks', max_branch=2)
+    result = b.compute(get=dask.get)
+    assert dict(result) == groupby(func, range(20))
+    # assert b.npartitions == 5
+
+
+def test_to_textfiles_empty_partitions():
+    with tmpdir() as d:
+        b = db.range(5, npartitions=5).filter(lambda x: x == 1).map(str)
+        b.to_textfiles(os.path.join(d, '*.txt'))
+        assert len(os.listdir(d)) == 5
