@@ -9,17 +9,26 @@ import types
 PY3 = sys.version_info[0] == 3
 PY2 = sys.version_info[0] == 2
 
+LZMA_AVAILABLE = True
+
 if PY3:
     import builtins
     from queue import Queue, Empty
     from itertools import zip_longest
     from io import StringIO, BytesIO
     from bz2 import BZ2File
-    from gzip import GzipFile
-    from lzma import LZMAFile
+    from gzip import (GzipFile, compress as gzip_compress,
+            decompress as gzip_decompress)
+    try:
+        from lzmaffi import (LZMAFile, compress as lzma_compress,
+                             decompress as lzma_decompress)
+    except ImportError:
+        from lzma import (LZMAFile, compress as lzma_compress,
+                          decompress as lzma_decompress)
     from urllib.request import urlopen
     from urllib.parse import urlparse
     from urllib.parse import quote, unquote
+    FileNotFoundError = FileNotFoundError
     unicode = str
     long = int
     def apply(func, args, kwargs=None):
@@ -51,10 +60,26 @@ else:
     range = xrange
     reduce = reduce
     operator_div = operator.div
+    FileNotFoundError = IOError
 
     def _getargspec(func):
         return inspect.getargspec(func)
 
+    def gzip_decompress(b):
+        f = gzip.GzipFile(fileobj=BytesIO(b))
+        result = f.read()
+        f.close()
+        return result
+
+    def gzip_compress(b):
+        bio = BytesIO()
+        f = gzip.GzipFile(fileobj=bio, mode='w')
+        f.write(b)
+        f.close()
+        bio.seek(0)
+        result = bio.read()
+        return result
+
     if sys.version_info[1] <= 7:
         class BZ2File(BufferedIOBase):
             def __init__(self, *args, **kwargs):
@@ -171,12 +196,19 @@ else:
         GzipFile = gzip.GzipFile
 
     try:
-        from backports.lzma import LZMAFile
+        try:
+            from lzmaffi import (LZMAFile, compress as lzma_compress,
+                                 decompress as lzma_decompress)
+        except ImportError:
+            from backports.lzma import LZMAFile
+            from backports.lzma import (LZMAFile, compress as lzma_compress,
+                                        decompress as lzma_decompress)
     except ImportError:
         class LZMAFile:
             def __init__(self, *args, **kwargs):
                 raise ValueError("xz files requires the lzma module. "
-                                 "To use, install backports.lzma.")
+                                 "To use, install lzmaffi or backports.lzma.")
+        LZMA_AVAILABLE = False
 
 
 def getargspec(func):
