@@ -1,9 +1,10 @@
 from __future__ import absolute_import, division, print_function
 
 import bisect
-from collections import Iterable, Iterator
+from collections import Iterator
 from datetime import datetime
 from distutils.version import LooseVersion
+import math
 import operator
 from operator import getitem, setitem
 from pprint import pformat
@@ -24,11 +25,12 @@ from .. import array as da
 from .. import core
 from ..array.core import partial_by_order
 from .. import threaded
-from ..compatibility import unicode, apply, operator_div, bind_method
+from ..compatibility import apply, operator_div, bind_method
 from ..utils import (repr_long_list, IndexCallable,
-                     pseudorandom, derived_from, different_seeds)
+                     pseudorandom, derived_from, different_seeds, funcname)
 from ..base import Base, compute, tokenize, normalize_token
 from ..async import get_sync
+from .utils import nonempty_sample_df
 
 no_default = '__no_default__'
 return_scalar = '__return_scalar__'
@@ -42,8 +44,6 @@ def _concat(args, **kwargs):
         return args
     if isinstance(first(core.flatten(args)), np.ndarray):
         return da.core.concatenate3(args)
-    if len(args) == 1:
-        return args[0]
     if isinstance(args[0], (pd.DataFrame, pd.Series)):
         args2 = [arg for arg in args if len(arg)]
         if not args2:
@@ -72,7 +72,7 @@ class Scalar(Base):
 
     _optimize = staticmethod(optimize)
     _default_get = staticmethod(threaded.get)
-    _finalize = staticmethod(finalize)
+    _finalize = staticmethod(first)
 
     def __init__(self, dsk, _name, name=None, divisions=None):
         self.dask = dsk
@@ -101,7 +101,7 @@ class Scalar(Base):
     @classmethod
     def _get_unary_operator(cls, op):
         def f(self):
-            name = tokenize(self)
+            name = funcname(op) + '-' + tokenize(self)
             dsk = {(name, 0): (op, (self._name, 0))}
             return Scalar(merge(dsk, self.dask), name)
         return f
@@ -112,7 +112,7 @@ class Scalar(Base):
 
 
 def _scalar_binary(op, a, b, inv=False):
-    name = '{0}-{1}'.format(op.__name__, tokenize(a, b))
+    name = '{0}-{1}'.format(funcname(op), tokenize(a, b))
 
     dsk = a.dask
     if not isinstance(b, Base):
@@ -216,6 +216,16 @@ class _Frame(Base):
     def _keys(self):
         return [(self._name, i) for i in range(self.npartitions)]
 
+    def __repr__(self):
+        name = self._name if len(self._name) < 10 else self._name[:7] + '...'
+        if self.known_divisions:
+            div_text = ', divisions=%s' % repr_long_list(self.divisions)
+        else:
+            div_text = ''
+
+        return ("dd.%s<%s, npartitions=%s%s>" %
+                (self.__class__.__name__, name, self.npartitions, div_text))
+
     @property
     def index(self):
         """Return dask Index instance"""
@@ -380,10 +390,13 @@ class _Frame(Base):
         """ Helper function for the .loc accessor """
         if isinstance(ind, Series):
             return self._loc_series(ind)
-        elif isinstance(ind, slice):
-            return self._loc_slice(ind)
+        if self.known_divisions:
+            if isinstance(ind, slice):
+                return self._loc_slice(ind)
+            else:
+                return self._loc_element(ind)
         else:
-            return self._loc_element(ind)
+            return map_partitions(try_loc, self, self, ind)
 
     def _loc_series(self, ind):
         if not self.divisions == ind.divisions:
@@ -392,21 +405,16 @@ class _Frame(Base):
                               self._pd, self, ind, token='loc-series')
 
     def _loc_element(self, ind):
-        name = 'loc-element-%s-%s' % (str(ind), self._name)
+        name = 'loc-%s' % tokenize(ind, self)
         part = _partition_of_index_value(self.divisions, ind)
         if ind < self.divisions[0] or ind > self.divisions[-1]:
             raise KeyError('the label [%s] is not in the index' % str(ind))
-        dsk = {(name, 0): (lambda df: df.loc[ind], (self._name, part))}
+        dsk = {(name, 0): (lambda df: df.loc[ind:ind], (self._name, part))}
 
-        if self.ndim == 1:
-            columns = self.name
-        else:
-            columns = ind
-        return self._constructor_sliced(merge(self.dask, dsk), name,
-                                        columns, [ind, ind])
+        return self._constructor(merge(self.dask, dsk), name, self, [ind, ind])
 
     def _loc_slice(self, ind):
-        name = 'loc-slice-%s-%s' % (str(ind), self._name)
+        name = 'loc-%s' % tokenize(ind, self)
         assert ind.step in (None, 1)
         if ind.start:
             start = _partition_of_index_value(self.divisions, ind.start)
@@ -448,22 +456,19 @@ class _Frame(Base):
         >>> df.loc["b":"d"]  # doctest: +SKIP"""
         return IndexCallable(self._loc)
 
-    @property
-    def iloc(self):
-        """ Not implemented """
+    # NOTE: `iloc` is not implemented because of performance concerns.
+    # see https://github.com/dask/dask/pull/507
 
-        # not implemented because of performance concerns.
-        # see https://github.com/dask/dask/pull/507
-        raise NotImplementedError("Dask Dataframe does not support iloc")
-
-    def repartition(self, divisions, force=False):
+    def repartition(self, divisions=None, npartitions=None, force=False):
         """ Repartition dataframe along new divisions
 
         Parameters
         ----------
-
         divisions : list
             List of partitions to be used
+        npartitions : int
+            Number of partitions of output, must be less than npartitions of
+            input
         force : bool, default False
             Allows the expansion of the existing divisions.
             If False then the new divisions lower and upper bounds must be
@@ -471,10 +476,18 @@ class _Frame(Base):
 
         Examples
         --------
-
-        >>> df = df.repartition([0, 5, 10, 20])  # doctest: +SKIP
+        >>> df = df.repartition(npartitions=10)  # doctest: +SKIP
+        >>> df = df.repartition(divisions=[0, 5, 10, 20])  # doctest: +SKIP
         """
-        return repartition(self, divisions, force=force)
+        if npartitions is not None:
+            if npartitions > self.npartitions:
+                raise ValueError("Can only repartition to fewer partitions")
+            return repartition_npartitions(self, npartitions)
+        elif divisions is not None:
+            return repartition(self, divisions, force=force)
+        else:
+            raise ValueError(
+                "Provide either divisions= or npartitions= to repartition")
 
     def __getstate__(self):
         return self.__dict__
@@ -534,12 +547,16 @@ class _Frame(Base):
         return to_csv(self, filename, get=get, **kwargs)
 
     def to_imperative(self):
+        warnings.warn("Deprecation warning: moved to to_delayed")
+        return self.to_delayed()
+
+    def to_delayed(self):
         """ Convert dataframe into dask Values
 
         Returns a list of values, one value per partition.
         """
-        from ..imperative import Value
-        return [Value(k, [self.dask]) for k in self._keys()]
+        from ..delayed import Delayed
+        return [Delayed(k, [self.dask]) for k in self._keys()]
 
     @classmethod
     def _get_unary_operator(cls, op):
@@ -552,58 +569,96 @@ class _Frame(Base):
         else:
             return lambda self, other: elemwise(op, self, other)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         raise NotImplementedError
 
+    def rolling(self, window, min_periods=None, win_type=None, axis=0):
+        """Provides rolling transformations.
+
+        Parameters
+        ----------
+
+        window: int
+            Size of the moving window. This window must be smaller
+            than the size of the previous partition.
+        min_periods: int, default None
+            Minimum number of observations in window required to have
+            a value (otherwise result is NA).
+        win_type: string, default None
+            Provide a window type. (Identical to pandas.)
+        axis: int, default 0
+            Provide the axis to apply the function. (Identical to pandas.)
+
+        The center argument and deprecated freq argument are not supported.
+        """
+        from dask.dataframe.rolling import Rolling
+
+        if not isinstance(window, int):
+            raise ValueError('window must be an integer')
+        if window < 0:
+            raise ValueError('window must be >= 0')
+
+        if min_periods is not None:
+            if not isinstance(min_periods, int):
+                raise ValueError('min_periods must be an integer')
+            if min_periods < 0:
+                raise ValueError('min_periods must be >= 0')
+
+        return Rolling(self, window=window, min_periods=min_periods,
+                       axis=axis, win_type=win_type)
+
     @derived_from(pd.DataFrame)
     def sum(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.sum(axis=1, skipna=skipna)
-            name = '{0}sum(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_sum, None, self,
+                                  token=self._token_prefix + 'sum',
+                                  axis=axis, skipna=skipna)
         else:
-            return self._aca_agg(token='sum', func=lambda x: x.sum(skipna=skipna))
+            return self._aca_agg(token='sum', func=_sum,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def max(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.max(axis=1, skipna=skipna)
-            name = '{0}max(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_max, None, self,
+                                  token=self._token_prefix + 'max',
+                                  skipna=skipna, axis=axis)
         else:
-            return self._aca_agg(token='max', func=lambda x: x.max(skipna=skipna))
+            return self._aca_agg(token='max', func=_max,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def min(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.min(axis=1, skipna=skipna)
-            name = '{0}min(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_min, None, self,
+                                  token=self._token_prefix + 'min',
+                                  skipna=skipna, axis=axis)
         else:
-            return self._aca_agg(token='min', func=lambda x: x.min(skipna=skipna))
+            return self._aca_agg(token='min', func=_min,
+                                 skipna=skipna, axis=axis)
 
     @derived_from(pd.DataFrame)
     def count(self, axis=None):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.count(axis=1)
-            name = '{0}count(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_count, None, self,
+                                  token=self._token_prefix + 'count',
+                                  axis=axis)
         else:
-            return self._aca_agg(token='count', func=lambda x: x.count(),
+            return self._aca_agg(token='count', func=_count,
                                  aggfunc=lambda x: x.sum())
 
     @derived_from(pd.DataFrame)
     def mean(self, axis=None, skipna=True):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.mean(axis=1, skipna=skipna)
-            name = '{0}mean(axis=1)'.format(self._token_prefix)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_mean, None, self,
+                                  token=self._token_prefix + 'mean',
+                                  axis=axis, skipna=skipna)
         else:
             num = self._get_numeric_data()
             s = num.sum(skipna=skipna)
@@ -614,16 +669,16 @@ class _Frame(Base):
                     return s / n
                 except ZeroDivisionError:
                     return np.nan
-            name = '{0}mean-{1}'.format(self._token_prefix, tokenize(s))
+            name = self._token_prefix + 'mean-%s' % tokenize(self, axis, skipna)
             return map_partitions(f, None, s, n, token=name)
 
     @derived_from(pd.DataFrame)
     def var(self, axis=None, skipna=True, ddof=1):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.var(axis=1, skipna=skipna, ddof=ddof)
-            name = '{0}var(axis=1, ddof={1})'.format(self._token_prefix, ddof)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_var, None, self,
+                                  token=self._token_prefix + 'var',
+                                  axis=axis, skipna=skipna, ddof=ddof)
         else:
             num = self._get_numeric_data()
             x = 1.0 * num.sum(skipna=skipna)
@@ -638,19 +693,20 @@ class _Frame(Base):
                     return result
                 except ZeroDivisionError:
                     return np.nan
-            name = '{0}var(ddof={1})'.format(self._token_prefix, ddof)
+            name = self._token_prefix + 'var-%s' % tokenize(self, axis, skipna, ddof)
             return map_partitions(f, None, x2, x, n, token=name)
 
     @derived_from(pd.DataFrame)
     def std(self, axis=None, skipna=True, ddof=1):
         axis = self._validate_axis(axis)
         if axis == 1:
-            f = lambda x: x.std(axis=1, skipna=skipna, ddof=ddof)
-            name = '{0}std(axis=1, ddof={1})'.format(self._token_prefix, ddof)
-            return map_partitions(f, None, self, token=name)
+            return map_partitions(_std, None, self,
+                                  token=self._token_prefix + 'std',
+                                  axis=axis, skipna=skipna, ddof=ddof)
         else:
             v = self.var(skipna=skipna, ddof=ddof)
-            name = '{0}std(ddof={1})'.format(self._token_prefix, ddof)
+            name = self._token_prefix + 'std-finish--%s' % tokenize(self, axis,
+                                                            skipna, ddof)
             return map_partitions(np.sqrt, None, v, token=name)
 
     def quantile(self, q=0.5, axis=0):
@@ -662,7 +718,7 @@ class _Frame(Base):
         q : list/array of floats, default 0.5 (50%)
             Iterable of numbers ranging from 0 to 1 for the desired quantiles
         axis : {0, 1, 'index', 'columns'} (default 0)
-            0 or 'index' for row-wise, 1 or 'columns' for column-wis
+            0 or 'index' for row-wise, 1 or 'columns' for column-wise
         """
         axis = self._validate_axis(axis)
         name = 'quantiles-concat--' + tokenize(self, q, axis)
@@ -922,16 +978,17 @@ class Series(_Frame):
     def dtype(self):
         """ Return data type """
         if self._known_dtype:
-            return self.dtype
+            return self._pd.dtype
         else:
             self._pd, self._known_dtype = self._build_pd(self.head())
             return self._pd.dtype
 
     def __getattr__(self, key):
         if key == 'cat':
-            head = self.head()
-            if isinstance(head.dtype, pd.core.dtypes.CategoricalDtype):
-                return head.cat
+            # If unknown dtype, need to infer from head.
+            if not self._known_dtype:
+                self.dtype
+            return self._pd.cat
         raise AttributeError("'Series' object has no attribute %r" % key)
 
     @property
@@ -944,11 +1001,6 @@ class Series(_Frame):
     def nbytes(self):
         return reduction(self, lambda s: s.nbytes, np.sum, token='nbytes')
 
-    def __repr__(self):
-        return ("dd.%s<%s, divisions=%s>" %
-                (self.__class__.__name__, self._name,
-                 repr_long_list(self.divisions)))
-
     def __array__(self, dtype=None, **kwargs):
         x = np.array(self.compute())
         if dtype and x.dtype != dtype:
@@ -981,7 +1033,7 @@ class Series(_Frame):
 
     def __getitem__(self, key):
         if isinstance(key, Series) and self.divisions == key.divisions:
-            name = 'series-index-%s[%s]' % (self._name, key._name)
+            name = 'index-%s' % tokenize(self, key)
             dsk = dict(((name, i), (operator.getitem, (self._name, i),
                                                        (key._name, i)))
                         for i in range(self.npartitions))
@@ -1007,14 +1059,15 @@ class Series(_Frame):
         # convert to numeric axis
         return {None: 0, 'index': 0}.get(axis, axis)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         if aggfunc is None:
             aggfunc = func
 
         return aca([self], chunk=func,
-                   aggregate=lambda x: aggfunc(pd.Series(x)),
-                   columns=return_scalar, token=self._token_prefix + token)
+                   aggregate=lambda x, **kwargs: aggfunc(pd.Series(x), **kwargs),
+                   columns=return_scalar, token=self._token_prefix + token,
+                   **kwargs)
 
     @derived_from(pd.Series)
     def groupby(self, index, **kwargs):
@@ -1074,9 +1127,10 @@ class Series(_Frame):
         uniques : Series
         """
         # unique returns np.ndarray, it must be wrapped
-        chunk = lambda x: pd.Series(pd.Series.unique(x), name=self.name)
+        name = self.name
+        chunk = lambda x: pd.Series(pd.Series.unique(x), name=name)
         return aca(self, chunk=chunk, aggregate=chunk,
-                   columns=self.name, token='unique')
+                   columns=name, token='unique')
 
     @derived_from(pd.Series)
     def nunique(self):
@@ -1150,7 +1204,6 @@ class Series(_Frame):
 
     @derived_from(pd.Series)
     def to_frame(self, name=None):
-        _name = name if name is not None else self.name
         return map_partitions(pd.Series.to_frame, self._pd.to_frame(name), self, name)
 
     @classmethod
@@ -1184,7 +1237,7 @@ class Series(_Frame):
             If list is given, the result is a DataFrame which columns is
             specified list. Otherwise, the result is a Series which name is
             given scalar or None (no name). If name keyword is not given, dask
-            tries to infer the result type using its beggining of data. This
+            tries to infer the result type using its beginning of data. This
             inference may take some time and lead to unexpected result.
         args: tuple
             Positional arguments to pass to function in addition to the array/series
@@ -1266,11 +1319,11 @@ class Index(Series):
     @derived_from(pd.Index)
     def max(self):
         # it doesn't support axis and skipna kwds
-        return self._aca_agg(token='max', func=lambda x: x.max())
+        return self._aca_agg(token='max', func=_max)
 
     @derived_from(pd.Index)
     def min(self):
-        return self._aca_agg(token='min', func=lambda x: x.min())
+        return self._aca_agg(token='min', func=_min)
 
     def count(self):
         f = lambda x: pd.notnull(x).sum()
@@ -1319,6 +1372,11 @@ class DataFrame(_Frame):
     def _constructor(self):
         return DataFrame
 
+    @property
+    def _pd_nonempty(self):
+        """ A Pandas dataframe with the same metadata but fake data."""
+        return nonempty_sample_df(self._pd)
+
     @property
     def columns(self):
         return self._pd.columns
@@ -1333,12 +1391,10 @@ class DataFrame(_Frame):
         self._name = renamed._name
 
     def __getitem__(self, key):
-
+        name = 'getitem-%s' % tokenize(self, key)
         if np.isscalar(key):
             # error is raised from pandas
             dummy = self._pd[_extract_pd(key)]
-
-            name = '{0}.{1}'.format(self._name, key)
             dsk = dict(((name, i), (operator.getitem, (self._name, i), key))
                         for i in range(self.npartitions))
             return self._constructor_sliced(merge(self.dask, dsk), name,
@@ -1348,7 +1404,6 @@ class DataFrame(_Frame):
             # error is raised from pandas
             dummy = self._pd[_extract_pd(key)]
 
-            name = '%s[%s]' % (self._name, str(key))
             dsk = dict(((name, i), (operator.getitem,
                                     (self._name, i), (list, key)))
                         for i in range(self.npartitions))
@@ -1360,7 +1415,6 @@ class DataFrame(_Frame):
             if self.divisions != key.divisions:
                 from .multi import _maybe_align_partitions
                 self, key = _maybe_align_partitions([self, key])
-            name = 'series-slice-%s[%s]' % (self._name, key._name)
             dsk = dict(((name, i), (self._partition_type._getitem_array,
                                      (self._name, i),
                                      (key._name, i)))
@@ -1377,11 +1431,7 @@ class DataFrame(_Frame):
 
     def __dir__(self):
         return sorted(set(dir(type(self)) + list(self.__dict__) +
-                      list(self.columns)))
-
-    def __repr__(self):
-        return ("dd.DataFrame<%s, divisions=%s>" %
-                (self._name, repr_long_list(self.divisions)))
+                      list(filter(pd.compat.isidentifier, self.columns))))
 
     @property
     def ndim(self):
@@ -1398,9 +1448,41 @@ class DataFrame(_Frame):
             return self._pd.dtypes
 
     @derived_from(pd.DataFrame)
-    def set_index(self, other, drop=True, **kwargs):
-        from .shuffle import set_index
-        return set_index(self, other, drop=drop, **kwargs)
+    def notnull(self):
+        return self.map_partitions(pd.DataFrame.notnull)
+
+    @derived_from(pd.DataFrame)
+    def isnull(self):
+        return self.map_partitions(pd.DataFrame.isnull)
+
+    def set_index(self, other, drop=True, sorted=False, **kwargs):
+        """ Set the DataFrame index (row labels) using an existing column
+
+        This operation in dask.dataframe is expensive.  If the input column is
+        sorted then we accomplish the set_index in a single full read of that
+        column.  However, if the input column is not sorted then this operation
+        triggers a full shuffle, which can take a while and only works on a
+        single machine (not distributed).
+
+        Parameters
+        ----------
+        other: Series or label
+        drop: boolean, default True
+            Delete columns to be used as the new index
+        sorted: boolean, default False
+            Set to True if the new index column is already sorted
+
+        Examples
+        --------
+        >>> df.set_index('x')  # doctest: +SKIP
+        >>> df.set_index(d.x)  # doctest: +SKIP
+        >>> df.set_index(d.timestamp, sorted=True)  # doctest: +SKIP
+        """
+        if sorted:
+            return set_sorted_index(self, other, drop=drop, **kwargs)
+        else:
+            from .shuffle import set_index
+            return set_index(self, other, drop=drop, **kwargs)
 
     def set_partition(self, column, divisions, **kwargs):
         """ Set explicit divisions for new column index
@@ -1445,7 +1527,7 @@ class DataFrame(_Frame):
 
         # Figure out columns of the output
         df2 = self._pd.assign(**_extract_pd(kwargs))
-        return elemwise(_assign, self, *pairs)
+        return elemwise(_assign, self, *pairs, columns=df2)
 
     @derived_from(pd.DataFrame)
     def rename(self, index=None, columns=None):
@@ -1470,7 +1552,7 @@ class DataFrame(_Frame):
         The original docstring follows below:\n
         """ + pd.DataFrame.query.__doc__
 
-        name = '%s.query(%s)' % (self._name, expr)
+        name = 'query-%s' % tokenize(self, expr)
         if kwargs:
             name = name + '--' + tokenize(kwargs)
             dsk = dict(((name, i), (apply, pd.DataFrame.query,
@@ -1484,6 +1566,14 @@ class DataFrame(_Frame):
         return self._constructor(merge(dsk, self.dask), name,
                                  dummy, self.divisions)
 
+    @derived_from(pd.DataFrame)
+    def eval(self, expr, inplace=None, **kwargs):
+        if '=' in expr and inplace in (True, None):
+            raise NotImplementedError("Inplace eval not supported."
+            " Please use inplace=False")
+        meta = self._pd.eval(expr, inplace=inplace, **kwargs)
+        return self.map_partitions(_eval, meta, expr, inplace=inplace, **kwargs)
+
     @derived_from(pd.DataFrame)
     def dropna(self, how='any', subset=None):
         # for cloudpickle
@@ -1539,16 +1629,19 @@ class DataFrame(_Frame):
         # convert to numeric axis
         return {None: 0, 'index': 0, 'columns': 1}.get(axis, axis)
 
-    def _aca_agg(self, token, func, aggfunc=None):
+    def _aca_agg(self, token, func, aggfunc=None, **kwargs):
         """ Wrapper for aggregations """
         if aggfunc is None:
             aggfunc = func
 
+        def aggregate(x, **kwargs):
+            return x.groupby(level=0).apply(aggfunc, **kwargs)
+
         # groupby.aggregation doesn't support skipna,
         # using gropuby.apply(aggfunc) is a workaround to handle each group as df
-        return aca([self], chunk=func,
-                   aggregate=lambda x: x.groupby(level=0).apply(aggfunc),
-                   columns=None, token=self._token_prefix + token)
+        return aca([self], chunk=func, aggregate=aggregate,
+                   columns=None, token=self._token_prefix + token,
+                   **kwargs)
 
     @derived_from(pd.DataFrame)
     def drop(self, labels, axis=0):
@@ -1653,7 +1746,7 @@ class DataFrame(_Frame):
             If list is given, the result is a DataFrame which columns is
             specified list. Otherwise, the result is a Series which name is
             given scalar or None (no name). If name keyword is not given, dask
-            tries to infer the result type using its beggining of data. This
+            tries to infer the result type using its beginning of data. This
             inference may take some time and lead to unexpected result
         args : tuple
             Positional arguments to pass to function in addition to the array/series
@@ -1702,6 +1795,22 @@ class DataFrame(_Frame):
         empty = self._pd.astype(dtype)
         return map_partitions(pd.DataFrame.astype, empty, self, dtype=dtype)
 
+    def info(self):
+        """
+        Concise summary of a Dask DataFrame.
+        """
+        lines = list()
+        lines.append(str(type(self)))
+        lines.append('Data columns (total %d columns):' % len(self.columns))
+        dtypes = self.dtypes
+        space = max([len(k) for k in self.columns]) + 4
+        template = "%s%s"
+        for i, col in enumerate(self.columns):
+            dtype = dtypes.iloc[i]
+            lines.append(template % (('%s' % col)[:space].ljust(space), dtype))
+
+        print('\n'.join(lines))
+
 
 # bind operators
 for op in [operator.abs, operator.add, operator.and_, operator_div,
@@ -1815,7 +1924,7 @@ def elemwise(op, *args, **kwargs):
     """ Elementwise operation for dask.Dataframes """
     columns = kwargs.pop('columns', no_default)
 
-    _name = 'elemwise-' + tokenize(op, kwargs, *args)
+    _name = funcname(op) + '-' + tokenize(op, kwargs, *args)
 
     args = _maybe_from_pandas(args)
 
@@ -1829,16 +1938,18 @@ def elemwise(op, *args, **kwargs):
     other = [(i, arg) for i, arg in enumerate(args)
              if not isinstance(arg, (_Frame, Scalar))]
 
-    if other:
-        op2 = partial_by_order(op, other)
-    else:
-        op2 = op
 
     # adjust the key length of Scalar
     keys = [d._keys() *  n if isinstance(d, Scalar)
             else d._keys() for d in dasks]
 
-    dsk = dict(((_name, i), (op2,) + frs) for i, frs in enumerate(zip(*keys)))
+    if other:
+        dsk = dict(((_name, i),
+                   (apply, partial_by_order, list(frs),
+                     {'function': op, 'other': other}))
+                   for i, frs in enumerate(zip(*keys)))
+    else:
+        dsk = dict(((_name, i), (op,) + frs) for i, frs in enumerate(zip(*keys)))
     dsk = merge(dsk, *[d.dask for d in dasks])
 
     if columns is no_default:
@@ -1897,7 +2008,7 @@ def reduction(x, chunk, aggregate, token=None):
     token = token or 'reduction'
     a = '{0}--chunk-{1}'.format(token, token_key)
     dsk = dict(((a, i), (empty_safe, chunk, (x._name, i)))
-                for i in range(x.npartitions))
+               for i in range(x.npartitions))
 
     b = '{0}--aggregation-{1}'.format(token, token_key)
     dsk2 = {(b, 0): (aggregate, (remove_empties,
@@ -1914,7 +2025,8 @@ def _maybe_from_pandas(dfs):
 
 
 def apply_concat_apply(args, chunk=None, aggregate=None,
-                       columns=no_default, token=None):
+                       columns=no_default, token=None, chunk_kwargs=None,
+                       aggregate_kwargs=None, **kwargs):
     """ Apply a function to blocks, the concat, then apply again
 
     Parameters
@@ -1936,6 +2048,13 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
 
     >>> apply_concat_apply([a, b], chunk=chunk, aggregate=agg)  # doctest: +SKIP
     """
+    if chunk_kwargs is None:
+        chunk_kwargs = dict()
+    if aggregate_kwargs is None:
+        aggregate_kwargs = dict()
+    chunk_kwargs.update(kwargs)
+    aggregate_kwargs.update(kwargs)
+
     if not isinstance(args, (tuple, list)):
         args = [args]
 
@@ -1946,19 +2065,22 @@ def apply_concat_apply(args, chunk=None, aggregate=None,
     token = token or 'apply-concat-apply'
 
     a = '{0}--first-{1}'.format(token, token_key)
-    if len(args) == 1 and isinstance(args[0], _Frame):
+    if len(args) == 1 and isinstance(args[0], _Frame) and not chunk_kwargs:
         dsk = dict(((a, i), (chunk, key))
                    for i, key in enumerate(args[0]._keys()))
     else:
         dsk = dict(((a, i), (apply, chunk, [(x._name, i)
                                             if isinstance(x, _Frame)
-                                            else x for x in args]))
+                                            else x for x in args],
+                                    kwargs))
                for i in range(args[0].npartitions))
 
     b = '{0}--second-{1}'.format(token, token_key)
-    dsk2 = {(b, 0): (aggregate,
-                      (_concat,
-                        (list, [(a, i) for i in range(args[0].npartitions)])))}
+    conc = (_concat,(list, [(a, i) for i in range(args[0].npartitions)]))
+    if not aggregate_kwargs:
+        dsk2 = {(b, 0): (aggregate, conc)}
+    else:
+        dsk2 = {(b, 0): (apply, aggregate, [conc], aggregate_kwargs)}
 
     if columns is no_default:
         return_type = type(args[0])
@@ -2009,7 +2131,7 @@ def _get_return_type(arg, metadata):
 
 def _extract_pd(x):
     """
-    Extract intenal cache data (``_pd``) from dd.DataFrame / dd.Series
+    Extract internal cache data (``_pd``) from dd.DataFrame / dd.Series
     """
     if isinstance(x, _Frame):
         return x._pd
@@ -2034,22 +2156,23 @@ def _emulate(func, *args, **kwargs):
     return func(*_extract_pd(args), **_extract_pd(kwargs))
 
 
-def map_partitions(func, columns, *args, **kwargs):
+def map_partitions(func, metadata, *args, **kwargs):
     """ Apply Python function on each DataFrame block
 
     Parameters
     ----------
 
-    column_info : tuple or string
-        Column names or name of the output
+    metadata: _Frame, columns, name
+        Metadata for output
     targets : list
         List of target DataFrame / Series.
     """
+    metadata = _extract_pd(metadata)
 
     assert callable(func)
-    token = kwargs.pop('token', 'map-partitions')
-    token_key = tokenize(token or func, columns, kwargs, *args)
-    name = '{0}-{1}'.format(token, token_key)
+    token = kwargs.pop('token', None)
+    name = token or funcname(func)
+    name = '{0}-{1}'.format(name, tokenize(metadata, *args, **kwargs))
 
     if all(isinstance(arg, Scalar) for arg in args):
         dask = {(name, 0):
@@ -2061,14 +2184,20 @@ def map_partitions(func, columns, *args, **kwargs):
     args = _maybe_align_partitions(args)
     dfs = [df for df in args if isinstance(df, _Frame)]
 
-    if columns is no_default:
+    if metadata is no_default:
         # pass no_default as much, because it updates internal cache
         try:
-            columns = _emulate(func, *args, **kwargs)
+            metadata = _emulate(func, *args, **kwargs)
         except Exception:
             # user function may fail
-            columns = None
-    metadata = columns
+            metadata = None
+
+    if isinstance(metadata, pd.DataFrame):
+        columns = metadata.columns
+    elif isinstance(metadata, pd.Series):
+        columns = metadata.name
+    else:
+        columns = metadata
 
     return_type = _get_return_type(dfs[0], metadata)
 
@@ -2077,7 +2206,7 @@ def map_partitions(func, columns, *args, **kwargs):
         values = [(arg._name, i if isinstance(arg, _Frame) else 0)
                   if isinstance(arg, (_Frame, Scalar)) else arg for arg in args]
         values = (apply, func, (tuple, values), kwargs)
-        dsk[(name, i)] = (_rename, metadata, values)
+        dsk[(name, i)] = (_rename, columns, values)
 
     dasks = [arg.dask for arg in args if isinstance(arg, (_Frame, Scalar))]
     return return_type(merge(dsk, *dasks), name, metadata, args[0].divisions)
@@ -2168,12 +2297,14 @@ def quantile(df, q):
     # currently, only Series has quantile method
     if isinstance(q, (list, tuple, np.ndarray)):
         # Index.quantile(list-like) must be pd.Series, not pd.Index
-        merge_type = lambda v: pd.Series(v, index=q, name=df.name)
+        df_name = df.name
+        merge_type = lambda v: pd.Series(v, index=q, name=df_name)
         return_type = df._constructor
         if issubclass(return_type, Index):
             return_type = Series
     else:
-        merge_type = lambda v: df._partition_type(v).item()
+        typ = df._partition_type
+        merge_type = lambda v: typ(v).item()
         return_type = df._constructor_sliced
         q = [q]
 
@@ -2326,7 +2457,7 @@ def pd_split(df, p, random_state=None):
 def _take_last(a, skipna=True):
     """
     take last row (Series) of DataFrame / last value of Seriese
-    considerning NaN.
+    considering NaN.
 
     Parameters
     ----------
@@ -2450,7 +2581,7 @@ def repartition_divisions(a, b, name, out1, out2, force=False):
         k += 1
 
     # right part of new division can remain
-    if a[-1] < b[-1]:
+    if a[-1] < b[-1] or b[-1] == b[-2]:
         for _j in range(j, len(b)):
             # always use right-most of old division
             # because it may contain last element
@@ -2479,12 +2610,12 @@ def repartition_divisions(a, b, name, out1, out2, force=False):
         while c[i] < b[j]:
             tmp.append((out1, i))
             i += 1
-        if last_elem and c[i] == b[-1] and i < k:
+        if last_elem and c[i] == b[-1] and (b[-1] != b[-2] or j == len(b) - 1) and i < k:
             # append if last split is not included
             tmp.append((out1, i))
             i += 1
         if len(tmp) == 0:
-            # dumy slice to return empty DataFrame or Series,
+            # dummy slice to return empty DataFrame or Series,
             # which retain original data attributes (columns / name)
             d[(out2, j - 1)] = (_loc, (name, 0), a[0], a[0], False)
         elif len(tmp) == 1:
@@ -2499,7 +2630,17 @@ def repartition_divisions(a, b, name, out1, out2, force=False):
     return d
 
 
-def repartition(df, divisions, force=False):
+def repartition_npartitions(df, npartitions):
+    """ Repartition dataframe to a smaller number of partitions """
+    npartitions = min(npartitions, df.npartitions)
+    k = int(math.ceil(df.npartitions / npartitions))
+    divisions = df.divisions[::k]
+    if len(divisions) <= npartitions:
+        divisions = divisions + (df.divisions[-1],)
+    return df.repartition(divisions=divisions)
+
+
+def repartition(df, divisions=None, force=False):
     """ Repartition dataframe along new divisions
 
     Dask.DataFrame objects are partitioned along their index.  Often when
@@ -2519,7 +2660,6 @@ def repartition(df, divisions, force=False):
         If False then the new divisions lower and upper bounds must be
         the same as the old divisions.
 
-
     Examples
     --------
 
@@ -2610,3 +2750,71 @@ class StringAccessor(Accessor):
     @staticmethod
     def call(obj, attr, *args):
         return getattr(obj.str, attr)(*args)
+
+
+def try_loc(df, ind):
+    try:
+        return df.loc[ind]
+    except KeyError:
+        return df.head(0)
+
+
+def set_sorted_index(df, index, drop=True, **kwargs):
+    if not isinstance(index, Series):
+        index2 = df[index]
+        meta = df._pd.set_index(index, drop=drop)
+    else:
+        index2 = index
+        meta = df._pd.set_index(index._pd, drop=drop)
+
+    mins = index2.map_partitions(pd.Series.min)
+    maxes = index2.map_partitions(pd.Series.max)
+    mins, maxes = compute(mins, maxes, **kwargs)
+
+    if (sorted(mins) != list(mins) or
+        sorted(maxes) != list(maxes) or
+        any(a >= b for a, b in zip(mins, maxes))):
+        raise ValueError("Column not properly sorted", mins, maxes)
+
+    divisions = tuple(mins) + (list(maxes)[-1],)
+
+    result = map_partitions(_set_sorted_index, meta, df, index, drop=drop)
+    result.divisions = divisions
+
+    return result
+
+
+def _set_sorted_index(df, idx, drop):
+    return df.set_index(idx, drop=drop)
+
+
+def _eval(df, expr, **kwargs):
+    return df.eval(expr, **kwargs)
+
+
+def _sum(x, **kwargs):
+    return x.sum(**kwargs)
+
+
+def _min(x, **kwargs):
+    return x.min(**kwargs)
+
+
+def _max(x, **kwargs):
+    return x.max(**kwargs)
+
+
+def _count(x, **kwargs):
+    return x.count(**kwargs)
+
+
+def _mean(x, **kwargs):
+    return x.mean(**kwargs)
+
+
+def _var(x, **kwargs):
+    return x.var(**kwargs)
+
+
+def _std(x, **kwargs):
+    return x.std(**kwargs)
