@@ -57,17 +57,17 @@ from __future__ import absolute_import, division, print_function
 
 from bisect import bisect_left, bisect_right
 
-from toolz import merge_sorted, unique, partial
+from toolz import merge_sorted, unique, partial, first
 import toolz
 import numpy as np
 import pandas as pd
 
 from ..base import tokenize
-from .core import (_get_return_type, _Frame, Scalar, DataFrame, Series,
+from ..compatibility import apply
+from .core import (_Frame, Scalar, DataFrame, map_partitions,
                    Index, _maybe_from_pandas)
 from .io import from_pandas
 from .shuffle import shuffle
-from . import utils
 
 
 def bound(seq, left, right):
@@ -203,8 +203,8 @@ def join_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix=''):
     (lhs, rhs), divisions, parts = align_partitions(lhs, rhs)
     divisions, parts = require(divisions, parts, required[how])
 
-    left_empty = lhs._pd
-    right_empty = rhs._pd
+    left_empty = lhs._pd_nonempty
+    right_empty = rhs._pd_nonempty
 
     name = 'join-indexed-' + tokenize(lhs, rhs, how, lsuffix, rsuffix)
 
@@ -247,7 +247,7 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
     """ Join two DataFrames on particular columns with hash join
 
     This shuffles both datasets on the joined column and then performs an
-    embarassingly parallel join partition-by-partition
+    embarrassingly parallel join partition-by-partition
 
     >>> hash_join(a, 'id', rhs, 'id', how='left', npartitions=10)  # doctest: +SKIP
     """
@@ -270,7 +270,7 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
         right_index = False
 
     # dummy result
-    dummy = pd.merge(lhs._pd, rhs._pd, how, None,
+    dummy = pd.merge(lhs._pd_nonempty, rhs._pd_nonempty, how, None,
                      left_on=left_on, right_on=right_on,
                      left_index=left_index, right_index=right_index,
                      suffixes=suffixes)
@@ -298,6 +298,25 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
                      name, dummy, divisions)
 
 
+def single_partition_join(left, right, **kwargs):
+    meta = pd.merge(left._pd_nonempty, right._pd_nonempty, **kwargs)
+    name = 'merge-' + tokenize(left, right, **kwargs)
+    if left.npartitions == 1:
+        left_key = first(left._keys())
+        dsk = dict(((name, i), (apply, pd.merge, [left_key, right_key],
+                                kwargs))
+                   for i, right_key in enumerate(right._keys()))
+        divisions = right.divisions
+    elif right.npartitions == 1:
+        right_key = first(right._keys())
+        dsk = dict(((name, i), (apply, pd.merge, [left_key, right_key],
+                                kwargs))
+                   for i, left_key in enumerate(left._keys()))
+        divisions = left.divisions
+    return DataFrame(toolz.merge(dsk, left.dask, right.dask), name,
+                     meta, divisions)
+
+
 def _pdconcat(dfs, axis=0, join='outer'):
     """ Concatenate caring empty Series """
 
@@ -398,7 +417,12 @@ def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
         return join_indexed_dataframes(left, right, how=how,
                                        lsuffix=suffixes[0], rsuffix=suffixes[1])
 
-    else:                           # Do hash join
+    elif (left.npartitions == 1 and how in ('inner', 'right') or
+          right.npartitions == 1 and how in ('inner', 'left')):
+        return single_partition_join(left, right, how=how, right_on=right_on,
+                left_on=left_on, left_index=left_index,
+                right_index=right_index, suffixes=suffixes)
+    else:
         return hash_join(left, left.index if left_index else left_on,
                          right, right.index if right_index else right_on,
                          how, npartitions, suffixes)
@@ -489,7 +513,7 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
     >>> b                                               # doctest: +SKIP
     dd.DataFrame<y, divisions=(2, 3, 6)>
     >>> dd.concat([a, b])                               # doctest: +SKIP
-    ValueError: All inputs have known divisions which cannnot be concatenated
+    ValueError: All inputs have known divisions which cannot be concatenated
     in order. Specify interleave_partitions=True to ignore order
 
     Specify interleave_partitions=True to ignore the division order.
@@ -545,7 +569,7 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
                 if interleave_partitions:
                     return concat_indexed_dataframes(dfs, join=join)
 
-                raise ValueError('All inputs have known divisions which cannnot '
+                raise ValueError('All inputs have known divisions which cannot '
                                  'be concatenated in order. Specify '
                                  'interleave_partitions=True to ignore order')
 
@@ -599,4 +623,3 @@ def melt(frame, id_vars=None, value_vars=None, var_name=None,
                                 value_vars=value_vars,
                                 var_name=var_name, value_name=value_name,
                                 col_level=col_level, token='melt')
-
