@@ -109,6 +109,9 @@ def test_attributes():
     assert 'foo' not in dir(d)
     assert raises(AttributeError, lambda: d.foo)
 
+    df = dd.from_pandas(pd.DataFrame({'a b c': [1, 2, 3]}), npartitions=2)
+    assert 'a b c' not in dir(df)
+
 
 def test_column_names():
     tm.assert_index_equal(d.columns, pd.Index(['a', 'b']))
@@ -610,7 +613,6 @@ def test_empty_quantile():
 
 
 def test_dataframe_quantile():
-
     # column X is for test column order and result division
     df = pd.DataFrame({'A': np.arange(20),
                        'X': np.arange(20, 40),
@@ -709,10 +711,6 @@ def test_loc_with_series():
     assert sorted(d.loc[d.a % 2].dask) != sorted(d.loc[d.a % 3].dask)
 
 
-def test_iloc_raises():
-    assert raises(NotImplementedError, lambda: d.iloc[:5])
-
-
 def test_getitem():
     df = pd.DataFrame({'A': [1, 2, 3, 4, 5, 6, 7, 8, 9],
                        'B': [9, 8, 7, 6, 5, 4, 3, 2, 1],
@@ -884,7 +882,7 @@ def test_concat4_interleave_partitions():
     ddf2 = dd.from_pandas(pdf2, 3)
     ddf3 = dd.from_pandas(pdf3, 2)
 
-    msg = ('All inputs have known divisions which cannnot be '
+    msg = ('All inputs have known divisions which cannot be '
            'concatenated in order. Specify '
            'interleave_partitions=True to ignore order')
 
@@ -1241,6 +1239,8 @@ def test_embarrassingly_parallel_operations():
 
     assert eq(a.x.notnull(), df.x.notnull())
     assert eq(a.x.isnull(), df.x.isnull())
+    assert eq(a.notnull(), df.notnull())
+    assert eq(a.isnull(), df.isnull())
 
     assert len(a.sample(0.5).compute()) < len(df)
 
@@ -1344,6 +1344,24 @@ def test_query():
         assert eq(q, df.query('x**2 > y'))
 
 
+@pytest.mark.skipif(LooseVersion(pd.__version__) <= '0.18.0',
+                    reason="eval inplace not supported")
+def test_eval():
+    p = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [5, 6, 7, 8]})
+    d = dd.from_pandas(p, npartitions=2)
+    with ignoring(ImportError):
+        assert eq(p.eval('x + y'), d.eval('x + y'))
+        assert eq(p.eval('z = x + y', inplace=False),
+                  d.eval('z = x + y', inplace=False))
+        with pytest.raises(NotImplementedError):
+            d.eval('z = x + y', inplace=True)
+
+        if p.eval('z = x + y', inplace=None) is None:
+            with pytest.raises(NotImplementedError):
+                d.eval('z = x + y', inplace=None)
+
+
+
 def test_deterministic_arithmetic_names():
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [5, 6, 7, 8]})
     a = dd.from_pandas(df, npartitions=2)
@@ -1453,7 +1471,7 @@ def test_apply():
     assert eq(ddf.apply(lambda xy: xy[0] + xy[1], axis='columns', columns=None),
               df.apply(lambda xy: xy[0] + xy[1], axis='columns'))
 
-    # inferrence
+    # inference
     assert eq(ddf.apply(lambda xy: xy[0] + xy[1], axis=1),
               df.apply(lambda xy: xy[0] + xy[1], axis=1))
     assert eq(ddf.apply(lambda xy: xy, axis=1),
@@ -1635,31 +1653,39 @@ def test_dataframe_itertuples():
 
 
 def test_from_delayed():
-    from dask import do
-    dfs = [do(tm.makeTimeDataFrame)(i) for i in range(1, 5)]
-    df = dd.from_delayed(dfs, columns=['A', 'B', 'C', 'D'])
+    from dask import delayed
+    dfs = [delayed(tm.makeTimeDataFrame)(i) for i in range(1, 5)]
+    df = dd.from_delayed(dfs, metadata=['A', 'B', 'C', 'D'])
 
     assert (df.compute().columns == df.columns).all()
     assert list(df.map_partitions(len).compute()) == [1, 2, 3, 4]
 
     ss = [df.A for df in dfs]
-    s = dd.from_delayed(ss, columns='A')
+    s = dd.from_delayed(ss, metadata='A')
 
     assert s.compute().name == s.name
     assert list(s.map_partitions(len).compute()) == [1, 2, 3, 4]
 
+    s = dd.from_delayed(ss)
+    assert s._known_dtype
+    assert s.compute().name == s.name
+
     df = dd.from_delayed(dfs, tm.makeTimeDataFrame(1))
     assert df._known_dtype
     assert list(df.columns) == ['A', 'B', 'C', 'D']
 
+    df = dd.from_delayed(dfs)
+    assert df._known_dtype
+    assert list(df.columns) == ['A', 'B', 'C', 'D']
+
 
 def test_to_delayed():
-    from dask.delayed import Value
+    from dask.delayed import Delayed
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [10, 20, 30, 40]})
     ddf = dd.from_pandas(df, npartitions=2)
     a, b = ddf.to_delayed()
-    assert isinstance(a, Value)
-    assert isinstance(b, Value)
+    assert isinstance(a, Delayed)
+    assert isinstance(b, Delayed)
 
     assert eq(a.compute(), df.iloc[:2])
 
@@ -1715,3 +1741,21 @@ def test_methods_tokenize_differently():
     df = dd.from_pandas(df, npartitions=1)
     assert (df.x.map_partitions(pd.Series.min)._name !=
             df.x.map_partitions(pd.Series.max)._name)
+
+
+def test_sorted_index_single_partition():
+    df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': [1, 0, 1, 0]})
+    ddf = dd.from_pandas(df, npartitions=1)
+    eq(ddf.set_index('x', sorted=True),
+        df.set_index('x'))
+
+
+def test_info(capsys):
+    df = pd.DataFrame({'long_column_name': [1, 2, 3, 4], 'short_name': [1, 0, 1, 0]})
+    ddf = dd.from_pandas(df, npartitions=1)
+    ddf.info()
+    out, err = capsys.readouterr()
+    assert out == ("<class 'dask.dataframe.core.DataFrame'>\n"
+                  "Data columns (total 2 columns):\n"
+                  "long_column_name    int64\n"
+                  "short_name          int64\n")
