@@ -110,6 +110,14 @@ def test_full_groupby():
     assert eq(d.groupby('a').apply(func), full.groupby('a').apply(func))
 
 
+def test_groupby_dir():
+    df = pd.DataFrame({'a': range(10), 'b c d e': range(10)})
+    ddf = dd.from_pandas(df, npartitions=2)
+    g = ddf.groupby('a')
+    assert 'a' in dir(g)
+    assert 'b c d e' not in dir(g)
+
+
 def test_groupby_on_index():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]},
                                   index=[0, 1, 3]),
@@ -161,6 +169,22 @@ def test_groupby_multilevel_getitem():
         assert eq(d.mean(), p.mean().astype(float))
 
 
+def test_groupby_multilevel_agg():
+    df = pd.DataFrame({'a': [1, 2, 3, 1, 2, 3],
+                       'b': [1, 2, 1, 4, 2, 1],
+                       'c': [1, 3, 2, 1, 1, 2],
+                       'd': [1, 2, 1, 1, 2, 2]})
+    ddf = dd.from_pandas(df, 2)
+
+    sol = df.groupby(['a']).mean()
+    res = ddf.groupby(['a']).mean()
+    assert eq(res, sol)
+
+    sol = df.groupby(['a', 'c']).mean()
+    res = ddf.groupby(['a', 'c']).mean()
+    assert eq(res, sol)
+
+
 def test_groupby_get_group():
     dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 6], 'b': [4, 2, 7]},
                                   index=[0, 1, 3]),
@@ -277,14 +301,11 @@ def test_groupby_set_index():
 
 
 def test_split_apply_combine_on_series():
-    dsk = {('x', 0): pd.DataFrame({'a': [1, 2, 6], 'b': [4, 2, 7]},
-                                  index=[0, 1, 3]),
-           ('x', 1): pd.DataFrame({'a': [4, 4, 6], 'b': [3, 3, 1]},
-                                  index=[5, 6, 8]),
-           ('x', 2): pd.DataFrame({'a': [4, 3, 7], 'b': [1, 1, 3]},
-                                  index=[9, 9, 9])}
-    ddf1 = dd.DataFrame(dsk, 'x', ['a', 'b'], [0, 4, 9, 9])
-    pdf1 = ddf1.compute()
+    pdf1 = pd.DataFrame({'a': [1, 2, 6, 4, 4, 6, 4, 3, 7],
+                         'b': [4, 2, 7, 3, 3, 1, 1, 1, 2]},
+                        index=[0, 1, 3, 5, 6, 8, 9, 9, 9])
+    ddf = dd.from_pandas(pdf1, npartitions=3)
+    ddf1 = ddf
 
     for ddkey, pdkey in [('b', 'b'), (ddf1.b, pdf1.b),
                          (ddf1.b + 1, pdf1.b + 1)]:
@@ -293,12 +314,22 @@ def test_split_apply_combine_on_series():
         assert eq(ddf1.groupby(ddkey).a.count(), pdf1.groupby(pdkey).a.count())
         assert eq(ddf1.groupby(ddkey).a.mean(), pdf1.groupby(pdkey).a.mean())
         assert eq(ddf1.groupby(ddkey).a.nunique(), pdf1.groupby(pdkey).a.nunique())
+        for ddof in [0, 1, 2]:
+            assert eq(ddf1.groupby(ddkey).a.var(ddof),
+                      pdf1.groupby(pdkey).a.var(ddof))
+            assert eq(ddf1.groupby(ddkey).a.std(ddof),
+                      pdf1.groupby(pdkey).a.std(ddof))
 
         assert eq(ddf1.groupby(ddkey).sum(), pdf1.groupby(pdkey).sum())
         assert eq(ddf1.groupby(ddkey).min(), pdf1.groupby(pdkey).min())
         assert eq(ddf1.groupby(ddkey).max(), pdf1.groupby(pdkey).max())
         assert eq(ddf1.groupby(ddkey).count(), pdf1.groupby(pdkey).count())
         assert eq(ddf1.groupby(ddkey).mean(), pdf1.groupby(pdkey).mean())
+        for ddof in [0, 1, 2]:
+            assert eq(ddf1.groupby(ddkey).var(ddof),
+                      pdf1.groupby(pdkey).var(ddof), check_dtype=False)
+            assert eq(ddf1.groupby(ddkey).std(ddof),
+                      pdf1.groupby(pdkey).std(ddof), check_dtype=False)
 
     for ddkey, pdkey in [(ddf1.b, pdf1.b), (ddf1.b + 1, pdf1.b + 1)]:
         assert eq(ddf1.a.groupby(ddkey).sum(), pdf1.a.groupby(pdkey).sum(), check_names=False)
@@ -306,6 +337,11 @@ def test_split_apply_combine_on_series():
         assert eq(ddf1.a.groupby(ddkey).count(), pdf1.a.groupby(pdkey).count(), check_names=False)
         assert eq(ddf1.a.groupby(ddkey).mean(), pdf1.a.groupby(pdkey).mean(), check_names=False)
         assert eq(ddf1.a.groupby(ddkey).nunique(), pdf1.a.groupby(pdkey).nunique(), check_names=False)
+        for ddof in [0, 1, 2]:
+            assert eq(ddf1.a.groupby(ddkey).var(ddof),
+                      pdf1.a.groupby(pdkey).var(ddof))
+            assert eq(ddf1.a.groupby(ddkey).std(ddof),
+                      pdf1.a.groupby(pdkey).std(ddof))
 
     for i in range(8):
         assert eq(ddf1.groupby(ddf1.b > i).a.sum(), pdf1.groupby(pdf1.b > i).a.sum())
@@ -334,6 +370,10 @@ def test_split_apply_combine_on_series():
         assert eq(ddf1.groupby(ddf1.a > i).count(), pdf1.groupby(pdf1.a > i).count())
         assert eq(ddf1.groupby(ddf1.a > i).mean(), pdf1.groupby(pdf1.a > i).mean())
 
+        for ddof in [0, 1, 2]:
+            assert eq(ddf1.groupby(ddf1.b > i).std(ddof),
+                      pdf1.groupby(pdf1.b > i).std(ddof))
+
     for ddkey, pdkey in [('a', 'a'), (ddf1.a, pdf1.a),
                          (ddf1.a + 1, pdf1.a + 1), (ddf1.a > 3, pdf1.a > 3)]:
         assert eq(ddf1.groupby(ddkey).b.sum(), pdf1.groupby(pdkey).b.sum())
@@ -349,6 +389,10 @@ def test_split_apply_combine_on_series():
         assert eq(ddf1.groupby(ddkey).count(), pdf1.groupby(pdkey).count())
         assert eq(ddf1.groupby(ddkey).mean(), pdf1.groupby(pdkey).mean().astype(float))
 
+        for ddof in [0, 1, 2]:
+            assert eq(ddf1.groupby(ddkey).b.std(ddof),
+                      pdf1.groupby(pdkey).b.std(ddof))
+
     assert sorted(ddf1.groupby('b').a.sum().dask) == \
            sorted(ddf1.groupby('b').a.sum().dask)
     assert sorted(ddf1.groupby(ddf1.a > 3).b.mean().dask) == \
@@ -366,6 +410,7 @@ def test_split_apply_combine_on_series():
     assert_dask_graph(ddf1.groupby('b').a.min(), 'series-groupby-min')
     assert_dask_graph(ddf1.groupby('b').a.max(), 'series-groupby-max')
     assert_dask_graph(ddf1.groupby('b').a.count(), 'series-groupby-count')
+    assert_dask_graph(ddf1.groupby('b').a.var(), 'series-groupby-var')
     # mean consists from sum and count operations
     assert_dask_graph(ddf1.groupby('b').a.mean(), 'series-groupby-sum')
     assert_dask_graph(ddf1.groupby('b').a.mean(), 'series-groupby-count')
