@@ -92,6 +92,29 @@ def test_resource_profiler():
             get(dsk, 'e')
 
 
+@pytest.mark.skipif("not psutil")
+def test_resource_profiler_multiple_gets():
+    with ResourceProfiler(dt=0.01) as rprof:
+        get(dsk2, 'c')
+        assert len(rprof.results) == 0
+        get(dsk2, 'c')
+    results = rprof.results
+    assert all(isinstance(i, tuple) and len(i) == 3 for i in results)
+
+    rprof.clear()
+    rprof.register()
+    get(dsk2, 'c')
+    assert len(rprof.results) > 0
+    get(dsk2, 'c')
+    rprof.unregister()
+
+    results = rprof.results
+    assert all(isinstance(i, tuple) and len(i) == 3 for i in results)
+
+    rprof.close()
+    assert not rprof._tracker.is_alive()
+
+
 def test_cache_profiler():
     with CacheProfiler() as cprof:
         out = get(dsk2, 'c')
@@ -118,7 +141,7 @@ def test_cache_profiler():
 @pytest.mark.skipif("not bokeh")
 def test_unquote():
     from dask.diagnostics.profile_visualize import unquote
-    from dask.imperative import to_task_dasks
+    from dask.delayed import to_task_dasks
     f = lambda x: to_task_dasks(x)[0]
     t = {'a': 1, 'b': 2, 'c': 3}
     assert unquote(f(t)) == t
