@@ -73,6 +73,21 @@ def tmpfile(extension='', dir=None):
                     os.remove(filename)
 
 
+@contextmanager
+def tmpdir(dir=None):
+    dirname = tempfile.mkdtemp(dir=dir)
+
+    try:
+        yield dirname
+    finally:
+        if os.path.exists(dirname):
+            if os.path.isdir(dirname):
+                shutil.rmtree(dirname)
+            else:
+                with ignoring(OSError):
+                    os.remove(dirname)
+
+
 @contextmanager
 def filetext(text, extension='', open=open, mode='w'):
     with tmpfile(extension=extension) as filename:
@@ -119,14 +134,14 @@ class IndexCallable(object):
 
 
 @contextmanager
-def filetexts(d, open=open):
+def filetexts(d, open=open, mode='t'):
     """ Dumps a number of textfiles to disk
 
     d - dict
         a mapping from filename to text like {'a.csv': '1,1\n2,2'}
     """
     for filename, text in d.items():
-        f = open(filename, 'wt')
+        f = open(filename, 'w' + mode)
         try:
             f.write(text)
         finally:
@@ -388,6 +403,10 @@ def file_size(fn, compression=None):
         with open(fn, 'rb') as f:
             f.seek(-4, 2)
             result = struct.unpack('I', f.read(4))[0]
+    elif compression:
+        # depending on the implementation, this may be inefficient
+        with open(fn, 'rb', compression) as f:
+            result = f.seek(0, 2)
     else:
         result = os.stat(fn).st_size
     return result
@@ -551,11 +570,58 @@ def derived_from(original_klass, version=None, ua_args=[]):
     return wrapper
 
 
-def funcname(func):
+def funcname(func, full=False):
     """Get the name of a function."""
     while hasattr(func, 'func'):
         func = func.func
     try:
-        return func.__name__
+        if full:
+            return func.__qualname__.strip('<>')
+        else:
+            return func.__name__.strip('<>')
     except:
-        return str(func)
+        return str(func).strip('<>')
+
+
+def ensure_bytes(s):
+    """ Turn string or bytes to bytes
+
+    >>> ensure_bytes(u'123')
+    '123'
+    >>> ensure_bytes('123')
+    '123'
+    >>> ensure_bytes(b'123')
+    '123'
+    """
+    if isinstance(s, bytes):
+        return s
+    if hasattr(s, 'encode'):
+        return s.encode()
+    raise TypeError(
+            "Object %s is neither a bytes object nor has an encode method" % s)
+
+
+def digit(n, k, base):
+    """
+
+    >>> digit(1234, 0, 10)
+    4
+    >>> digit(1234, 1, 10)
+    3
+    >>> digit(1234, 2, 10)
+    2
+    >>> digit(1234, 3, 10)
+    1
+    """
+    return n // base**k  % base
+
+
+def insert(tup, loc, val):
+    """
+
+    >>> insert(('a', 'b', 'c'), 0, 'x')
+    ('x', 'b', 'c')
+    """
+    L = list(tup)
+    L[loc] = val
+    return tuple(L)
